searchData={"items":[{"type":"module","title":"CliMate","doc":"This module is the base namespace for the `:cli_mate` application.\n\nMost interactions with this library will be made through the `CliMate.CLI`\nmodule.","ref":"CliMate.html"},{"type":"module","title":"Deprecation for `use CliMate` {: .warning} - CliMate","doc":"Including all CLI code in your own module is no longer supported and will be\nremovedin a future release.\n\nPlease see\nhttps://github.com/lud/cli_mate?tab=readme-ov-file#migration-to-version-100\n\nIf you still want to extend the CLI module to add your own helpers, you can use\nthe following:\n\n    require CliMate.CLI\n    CliMate.CLI.extend()\n\nThis will import and re-export all the CLI exported functions into your module.","ref":"CliMate.html#module-deprecation-for-use-climate-warning"},{"type":"module","title":"CliMate.CLI","doc":"Main API to interact with the command line.","ref":"CliMate.CLI.html"},{"type":"module","title":"Basic usage - CliMate.CLI","doc":"import CliMate.CLI\n\n    def run(argv) do\n      command = [options: [verbose: [type: :boolean]], arguments: [n: [type: :integer]]]\n      %{options: opts, arguments: args} = parse_or_halt!(argv, command)\n\n      if opts.verbose do\n        writeln(\"Hello!\")\n      end\n\n      case do_something_useful(args.n) do\n        :ok -> halt_success(\"Done!\")\n        {:error, reason} -> halt_error(reason)\n      end\n    end","ref":"CliMate.CLI.html#module-basic-usage"},{"type":"function","title":"CliMate.CLI.color/2","doc":"Wraps the given `iodata` with the given `color`.\n\n`color` should be a `IO.ANSI.format/2` compatible atom.","ref":"CliMate.CLI.html#color/2"},{"type":"function","title":"CliMate.CLI.debug/1","doc":"Outputs `iodata` in the current shell, formatted with cyan color.","ref":"CliMate.CLI.html#debug/1"},{"type":"function","title":"CliMate.CLI.error/1","doc":"Outputs `iodata` to `stderr` in the current shell, formatted with bright\nred color.","ref":"CliMate.CLI.html#error/1"},{"type":"macro","title":"CliMate.CLI.extend/0","doc":"Delegates all `CliMate.CLI` functions from the calling module.\n\nThis is useful if you want to define a module where all your CLI helpers\nreside, instead of calling, say, `writeln(\"hello\")` from\n`CliMate.CLI` but `fancy_subtitle(\"Hello!\")` from\n`MyApp.CliHelpers`.","ref":"CliMate.CLI.html#extend/0"},{"type":"function","title":"CliMate.CLI.format_usage/2","doc":"Returns a standard \"usage\" documentation block describing the different\noptions of the given command.","ref":"CliMate.CLI.html#format_usage/2"},{"type":"function","title":"Options - CliMate.CLI.format_usage/2","doc":"* `:format` - If `:moduledoc`, the formatted usage will be compatible for\n  embedding in a `@moduledoc` attribute. Any other value will generate a\n  simple terminal styled text. Defaults to `:cli`.\n* `:io_columns` - Number of columns for the terminal, defaults to a call to\n  `:io.columns/0`. Only used when format is `:cli`.","ref":"CliMate.CLI.html#format_usage/2-options"},{"type":"function","title":"CliMate.CLI.halt/1","doc":"Stops the execution of the Erlang runtime, with a given return code.\n\nIf not provided, the return code will be `0`.","ref":"CliMate.CLI.html#halt/1"},{"type":"function","title":"CliMate.CLI.halt_error/2","doc":"Combines `error/1` then `halt/1`. Halts the Erlang runtime with a `1`\nreturn code by default.","ref":"CliMate.CLI.html#halt_error/2"},{"type":"function","title":"CliMate.CLI.halt_success/1","doc":"Combines `success/1` then `halt/1`. Halts the Erlang runtime with a `0`\nreturn code.","ref":"CliMate.CLI.html#halt_success/1"},{"type":"function","title":"CliMate.CLI.parse/2","doc":"Accepts the command line arguments and the definition of a command (options,\narguments, and metadata) and returns a parse result with flues extracted from\nthe command line arguments.","ref":"CliMate.CLI.html#parse/2"},{"type":"function","title":"Defining options - CliMate.CLI.parse/2","doc":"Options definitions is a `Keyword` whose keys are the option name, and values\nare the options parameters. Keys with underscores like `some_thing` define\noptions in kebab case like `--some-thing`.\n\nThe available settings for an option are described in the\n`CliMate.CLI.Option` module.\n\nThe `:help` option is always defined and cannot be overridden.","ref":"CliMate.CLI.html#parse/2-defining-options"},{"type":"function","title":"Options examples - CliMate.CLI.parse/2","doc":"iex> {:ok, result} = parse(~w(--who joe), [options: [who: [type: :string]]])\n    iex> result.options.who\n    \"joe\"\n\n    iex> {:ok, result} = parse(~w(--who joe), [options: [who: []]])\n    iex> result.options.who\n    \"joe\"\n\n    iex> {:ok, result} = parse(~w(--port 4000), [options: [port: [type: :integer]]])\n    iex> result.options.port\n    4000\n\n    iex> {:ok, result} = parse(~w(-p 4000), [options: [port: [type: :integer, short: :p]]])\n    iex> result.options.port\n    4000\n\n    iex> parse(~w(--port nope), [options: [port: [type: :integer]]])\n    {:error, {:invalid, [{\"--port\", \"nope\"}]}}\n\n    iex> {:ok, result} = parse([], [options: [lang: [default: \"elixir\"]]])\n    iex> result.options.lang\n    \"elixir\"\n\n    iex> {:ok, result} = parse([], [options: [lang: []]])\n    iex> Map.has_key?(result.options, :lang)\n    false\n\n    iex> {:ok, result} = parse([], options: [])\n    iex> result.options.help\n    false\n\n    iex> {:ok, result} = parse(~w(--help), options: [])\n    iex> result.options.help\n    true","ref":"CliMate.CLI.html#parse/2-options-examples"},{"type":"function","title":"Defining arguments - CliMate.CLI.parse/2","doc":"Arguments can be defined in the same way as options, providing a `Keyword`\nwhere the keys are the argument names and the values are the parameters.\n\nThe available settings for an argument are described in the\n`CliMate.CLI.Argument` module.","ref":"CliMate.CLI.html#parse/2-defining-arguments"},{"type":"function","title":"Arguments examples - CliMate.CLI.parse/2","doc":"iex> {:ok, result} = parse(~w(joe), arguments: [who: []])\n    iex> result.arguments.who\n    \"joe\"\n\n    iex> parse([], arguments: [who: []])\n    {:error, {:missing_argument, :who}}\n\n    iex> {:ok, result} = parse([], arguments: [who: [required: false]])\n    iex> result.arguments\n    %{}\n\n    iex> cast = fn string -> Date.from_iso8601(string) end\n    iex> {:ok, result} = parse([\"2022-12-22\"], arguments: [date: [cast: cast]])\n    iex> result.arguments.date\n    ~D[2022-12-22]\n\n    iex> cast = {Date, :from_iso8601, []}\n    iex> {:ok, result} = parse([\"2022-12-22\"], arguments: [date: [cast: cast]])\n    iex> result.arguments.date\n    ~D[2022-12-22]\n\n    iex> cast = {Date, :from_iso8601, []}\n    iex> parse([\"not-a-date\"], arguments: [date: [cast: cast]])\n    {:error, {:argument_cast, :date, :invalid_format}}","ref":"CliMate.CLI.html#parse/2-arguments-examples"},{"type":"function","title":"CliMate.CLI.parse_or_halt!/2","doc":"Attempts to parse the command line arguments `argv` with the defined\ncommand.\n\nCommand options and arguments are documented in the `parse/2` function of\nthis module.\n\nIn `parse_or_halt!/2`, the successful return value will not be wrapped in\nan `:ok` tuple, but directly a map with the `:options` and `:arguments`\nkeys.\n\nIn case of a parse error, this function will output the usage block\nfollowed by a formatted error message, and halt the Erlang runtime.","ref":"CliMate.CLI.html#parse_or_halt!/2"},{"type":"function","title":"CliMate.CLI.put_shell/1","doc":"Defines the current shell to send console messages to. Accepts either\n`CliMate.CLI` or `CliMate.CLI.ProcessShell`.\n\nThis is mostly done for testing. The default shell outputs to standard io as\nyou would expect.\n\nThe shell is saved to `:persistent_term`, so shell should not be changed\nrepeatedly during runtime. This method of persistence is subject to change and\nshould not be relied on.","ref":"CliMate.CLI.html#put_shell/1"},{"type":"function","title":"CliMate.CLI.safe_to_string/1","doc":"Returns a string representation of the given term, with fallback to\n`Kernel.inspect/1`.","ref":"CliMate.CLI.html#safe_to_string/1"},{"type":"function","title":"CliMate.CLI.shell/0","doc":"Returns the current shell to send output to.","ref":"CliMate.CLI.html#shell/0"},{"type":"function","title":"CliMate.CLI.success/1","doc":"Outputs `iodata` in the current shell, formatted with green color.","ref":"CliMate.CLI.html#success/1"},{"type":"function","title":"CliMate.CLI.warn/1","doc":"Outputs `iodata` to `stderr` in the current shell, formatted with yellow\ncolor.","ref":"CliMate.CLI.html#warn/1"},{"type":"function","title":"CliMate.CLI.write/1","doc":"Outputs `iodata` in the current shell.","ref":"CliMate.CLI.html#write/1"},{"type":"function","title":"CliMate.CLI.writeln/1","doc":"Outputs `iodata` in the current shell.","ref":"CliMate.CLI.html#writeln/1"},{"type":"module","title":"CliMate.CLI.Argument","doc":"Describes an argument.\n\nWhen declaring a command, the `:arguments` entry is a keyword list that\naccepts these settings, all optional:\n\n* `:required` - A boolean. Arguments are required by default.\n* `:type` - `:string`, `:integer` or `:float`. Defaults to `:string`.\n* `:doc` - A string to document what the argument is for. This is not used at\n  the moment but future releases will exploit it.\n* `:cast` - A fun accepting the value, or a `{module, function, arguments}`,\n  returning a result tuple.  See the \"Casting\" section below for more\n  information.","ref":"CliMate.CLI.Argument.html"},{"type":"module","title":"Casting - CliMate.CLI.Argument","doc":"When the `:cast` option is a fun, it will be called with the deserialized\nvalue. When it is an \"MFA\" tuple, the deserialized value will be prepended to\nthe given arguments.\n\n#### Cast functions input\n\nWhat is a deserialized value though? By default it will be the raw string\npassed in the shell as command line arguments. But if the `:type` option is\nalso defined to `:integer` or `:float`, your cast function will be called with\nthe corresponding type. If that type deserialization fails, the cast function\nwill not be called.\n\n#### Returning errors\n\nCast functions must return `{:ok, value}` or `{:error, reason}`. The `reason`\ncan be anything, but at the moment CliMate doesn't do any special formatting\non it to display errors (besides calling `to_string/1` with a safe fallback to\n`inspect/1`). It is advised to return a meaningful error message as the\nreason.\n\n#### Compilation commands\n\nWhen commands are declared as a module attribute, you cannot use local\nfunctions definitions or the compilation will fail with \"undefined function\n...\". You need to use a remote form by prefixing by `__MODULE__`, or use an\nMFA tuple.\n\n    defmodule MyCommand do\n      use Mix.Task\n\n      @command name: \"my command\",\n               arguments: [\n                 some_arg: [\n                   cast: &__MODULE__.cast_some/1\n                 ]\n               ]\n    end","ref":"CliMate.CLI.Argument.html#module-casting"},{"type":"function","title":"CliMate.CLI.Argument.new/2","doc":"","ref":"CliMate.CLI.Argument.html#new/2"},{"type":"type","title":"CliMate.CLI.Argument.t/0","doc":"","ref":"CliMate.CLI.Argument.html#t:t/0"},{"type":"type","title":"CliMate.CLI.Argument.vtype/0","doc":"","ref":"CliMate.CLI.Argument.html#t:vtype/0"},{"type":"module","title":"CliMate.CLI.Option","doc":"Describes an option.\n\nWhen declaring a command, the `:options` entry is a keyword list that accepts\nthese settings, all optional:\n\n* `:type` - Uses the `OptionParser` module under the hood, so the following\n  types are accepted:\n  * `:boolean` - If the default value is `true`, `OptionParser` supports the\n    `--no-` prefix to the options.\n  * `:count` - Counts the number of times the flag is given.\n  * `:integer` - Value is parsed or an error is returned.\n  * `:float` - Same as `:integer`.\n  * `:string` - Value is used as-is.\n* `:doc` - A string describing the role of the option. It is displayed in the\n  usage block shown with `--help` or `mix help`.\n* `:short` - A single letter atom like `:a`, `:b`, etc. For instance with\n  these options:\n\n      @command options: [\n        name: [short: :n]\n      ]\n\n* `:default` - The default value to set if the option is not given. See the\n  \"Default values\" section below.\n\n  The command line will accept both `--name` and `-n` switches.\n\n* `:keep` - A boolean. This flag is used with `OptionParser`. When `true`\n  duplicate options will not override each other but will rather be\n  accumulated in a list. The option value will always be a list even when\n  provided a single time in the command line arguments. The option will also\n  be present, as an empty list, if the option is not given in command line\n  arguments.\n* `:doc_arg` - A string to display in the usage block. Defaults the type of\n  the option, as in:\n\n  ```text\n  Options\n\n    -n --some-int    This option does not define :doc_arg.\n    -o --other-int     Here \"other\" was provided as :doc_arg.\n  ```\n* `:default_doc` - The string to output when CliMate does not know how to\n  display the default value. That is, when the default value is defined by a\n  function. Not used when `:default` is not defined.","ref":"CliMate.CLI.Option.html"},{"type":"module","title":"Default values - CliMate.CLI.Option","doc":"Default values can be omitted, in that case, the option will not be present at\nall when parsing the command line arguments. This is especially important for\nbooleans, where one would expect that they automatically default to false. It\nis not the case.\n\nWhen defined, a default value can be:\n\n* A raw value, that is anything that is not a function. This value will be\n  used as the default value.\n* A function of arity zero. This function will be called when the option is\n  not provided in the command line and the result value will be used as the\n  default value. For instance `fn -> 123 end` or `&default_for_some_opt/0`.\n* A function of arity one. This function will be called with the option key as\n  its argument. For instance, passing `&default_opt/1` as the `:default` for\n  an option definition allow to define the following function:\n\n      defp default_opt(:port), do: 4000\n      defp default_opt(:scheme), do: \"http\"\n\nWhen the command is defined in a module attribute, you need to pass the module\nprefix or the compilation will fail:\n\n    defmodule MyCommand do\n      use Mix.Task\n\n      @command name: \"my command\",\n               options: [\n                 some_arg: [\n                   default: &__MODULE__.cast_some/1\n                 ]\n               ]\n    end","ref":"CliMate.CLI.Option.html#module-default-values"},{"type":"function","title":"CliMate.CLI.Option.new/2","doc":"","ref":"CliMate.CLI.Option.html#new/2"},{"type":"type","title":"CliMate.CLI.Option.t/0","doc":"","ref":"CliMate.CLI.Option.html#t:t/0"},{"type":"type","title":"CliMate.CLI.Option.vtype/0","doc":"","ref":"CliMate.CLI.Option.html#t:vtype/0"},{"type":"module","title":"CliMate.CLI.ProcessShell","doc":"An output shell implementation allowing your CLI commands to send their output\nas messages to themselves. This is most useful for tests.\n\nThe process that will receive the messages is found by looking up the first\npid in the `:\"$callers\"` key of the process dictionary, or `self()` if there\nis no caller.\n\nUse `CliMate.CLI.put_shell(CliMate.CLI.ProcessShell)` to enable this shell.\n\nThis shell is called when calling one of:\n\n* `CliMate.CLI.error/1\n* `CliMate.CLI.warn/1\n* `CliMate.CLI.debug/1\n* `CliMate.CLI.success/1\n* `CliMate.CLI.writeln/1\n* `CliMate.CLI.halt_success/1\n* `CliMate.CLI.halt_error/1\n* `CliMate.CLI.halt_error/2\n\nA message of type `t:shell_message/0` will be sent to `self()`, or the caller\nprocess for `Task.async/1` and similar task functions.","ref":"CliMate.CLI.ProcessShell.html"},{"type":"function","title":"CliMate.CLI.ProcessShell.build_message/2","doc":"","ref":"CliMate.CLI.ProcessShell.html#build_message/2"},{"type":"type","title":"CliMate.CLI.ProcessShell.kind/0","doc":"","ref":"CliMate.CLI.ProcessShell.html#t:kind/0"},{"type":"function","title":"CliMate.CLI.ProcessShell.message_target/0","doc":"Returns the pid of the process that will receive output messages.","ref":"CliMate.CLI.ProcessShell.html#message_target/0"},{"type":"type","title":"CliMate.CLI.ProcessShell.shell_message/0","doc":"","ref":"CliMate.CLI.ProcessShell.html#t:shell_message/0"},{"type":"task","title":"mix cli.embed","doc":"Copies the CLI code into your own application.","ref":"Mix.Tasks.Cli.Embed.html"},{"type":"task","title":"Synopsis - mix cli.embed","doc":"mix cli.embed [options]","ref":"Mix.Tasks.Cli.Embed.html#module-synopsis"},{"type":"task","title":"Arguments - mix cli.embed","doc":"* `prefix` - The root namespace for the generated modules.\n  Example: MyApp.CLI.\n* `path` - The base directory for the generated modules. When the --extend\n  option is not provided, the base module is definied as  .ex,\n  that is outside of said directory.","ref":"Mix.Tasks.Cli.Embed.html#module-arguments"},{"type":"task","title":"Options - mix cli.embed","doc":"* `--extend` - When true, the base CLI module will be defined as ` .Base`\n  and will export an extend/0 macro. You will have to define your\n  main CLI module and call `require( .Base).extend()` from\n  there.\n  \n  When false, the command will define the main CLI module as\n  ` ` directly. The `extend/0` macro is still included. Defaults to `false`.\n* `--moduledoc` - When true, include @moduledoc attributes in the generated code.\n  When false, defines `@moduledoc false` in all generated modules. Defaults to `true`.\n* `-f, --force` - Actually writes generated code to disk. Without this option the\n  command only prints debug information. Defaults to `false`.\n* `-y, --yes` - Automatically accept prompts to overwrite files. Defaults to `false`.\n* `--help` - Displays this help.","ref":"Mix.Tasks.Cli.Embed.html#module-options"},{"type":"task","title":"mix example","doc":"Formats an integer in base two or eight","ref":"Mix.Tasks.Example.html"},{"type":"task","title":"Synopsis - mix example","doc":"mix example [options]","ref":"Mix.Tasks.Example.html#module-synopsis"},{"type":"task","title":"Arguments - mix example","doc":"* `n` - The value to convert.\n* `base` - An numeric base. Accepts 'two' or 'eigh' only!","ref":"Mix.Tasks.Example.html#module-arguments"},{"type":"task","title":"Options - mix example","doc":"* `-v, --verbose` - Output debug info about the command. Defaults to `false`.\n* `--help` - Displays this help.","ref":"Mix.Tasks.Example.html#module-options"},{"type":"function","title":"Mix.Tasks.Example.cast_base/1","doc":"","ref":"Mix.Tasks.Example.html#cast_base/1"},{"type":"extras","title":"License","doc":"# The MIT License\n\nCopyright (c) 2022, Ludovic Demblans\n\nPermission is hereby granted, free of charge, to any person obtaining a copy of\nthis software and associated documentation files (the \"Software\"), to deal in\nthe Software without restriction, including without limitation the rights to\nuse, copy, modify, merge, publish, distribute, sublicense, and/or sell copies\nof the Software, and to permit persons to whom the Software is furnished to do\nso, subject to the following conditions:\n\nThe above copyright notice and this permission notice shall be included in all\ncopies or substantial portions of the Software.\n\nTHE SOFTWARE IS PROVIDED \"AS IS\", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR\nIMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,\nFITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE\nAUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER\nLIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,\nOUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE\nSOFTWARE.","ref":"license.html"},{"type":"extras","title":"Overview","doc":"# CliMate\n\nA lightweight, flexible Command-Line-Interface toolkit for Elixir applications.","ref":"readme.html"},{"type":"extras","title":"Features - Overview","doc":"* Command line options and arguments parser that extends [OptionParser](https://hexdocs.pm/elixir/OptionParser.html)\n* Shell helpers for formatted output in releases where `Mix` is unavailable\n* Automatic usage formatting for `--help` and `mix help` commands\n* Support for custom type casting and validation","ref":"readme.html#features"},{"type":"extras","title":"Documentation - Overview","doc":"Detailed documentation is available on [HexDocs](https://hexdocs.pm/cli_mate):\n\n* [Getting Started](https://hexdocs.pm/cli_mate/getting_started.html) - Getting started with CliMate","ref":"readme.html#documentation"},{"type":"extras","title":"Table of Contents - Overview","doc":"- [Features](#features)\n- [Documentation](#documentation)\n- [Table of Contents](#table-of-contents)\n- [Installation](#installation)\n- [Quick Example](#quick-example)\n- [Building CLI Applications in Elixir](#building-cli-applications-in-elixir)\n- [Migrating from older versions](#migrating-from-older-versions)\n- [Roadmap](#roadmap)\n- [License](#license)","ref":"readme.html#table-of-contents"},{"type":"extras","title":"Installation - Overview","doc":"```elixir\ndef deps do\n  [\n    {:cli_mate, \"~> 0.8\", runtime: false},\n  ]\nend\n```","ref":"readme.html#installation"},{"type":"extras","title":"Quick Example - Overview","doc":"```elixir\ndefmodule Mix.Tasks.Example do\n  import CliMate.CLI\n  use Mix.Task\n\n  @command name: \"mix example\",\n           module: __MODULE__,\n           options: [\n             verbose: [\n               short: :v,\n               type: :boolean,\n               default: false,\n               doc: \"Output debug info.\"\n             ]\n           ],\n           arguments: [\n             n: [type: :integer, doc: \"The value to process.\"]\n           ]\n\n  @impl true\n  def run(argv) do\n    command = parse_or_halt!(argv, @command)\n\n    if command.options.verbose do\n      writeln(\"Hello!\")\n    end\n\n    SomeModule.do_something_with(command.arguments.n)\n  end\nend\n```","ref":"readme.html#quick-example"},{"type":"extras","title":"Building CLI Applications in Elixir - Overview","doc":"Due to the startup time of the BEAM, it is generally discouraged to build\ncommand line applications with Elixir.\n\nWhile the startup problem is real, this is only important for small utilities\nlike `ls`, `grep` or `cat`. You surely do not want that delay when piping or\nlooping with those commands in bash scripts.\n\nBut for commands that are doing more, like deployments or asset bundling, or\ntools that run for a while like credo or dialyzer it is totally fine. And you\nget to write them with Elixir!","ref":"readme.html#building-cli-applications-in-elixir"},{"type":"extras","title":"Migrating from older versions - Overview","doc":"","ref":"readme.html#migrating-from-older-versions"},{"type":"extras","title":"Migration to version 0.8.0 - Overview","doc":"Support for installable mix tasks is back, but now relies on code generation.\n\nIf your library was intended to be installed like this:\n\n    mix archive.install hex your_library\n\nThis would not work when using CliMate because it would not embed the CLI code\nin your code directly on compilation. This was dropped because maintenance of a full library wrapped in a `quote do` block was not perennial.\n\nSince version 0.8.0, the `mix cli.embed` task will generate the CLI code\ndirectly into your library:\n\n    mix cli.embed MyApp.CLI lib/my_app/cli\n\nMake sure to read the different options by calling `mix help cli.embed`.\n\nWhen using code generation, CliMate should now be used as a dev dependency:\n\n```elixir\ndef deps do\n  [\n    {:cli_mate, \"~> 0.8\", only: [:dev, :test], runtime: false},\n  ]\nend\n```\n\nNote that this is fully optional. If you are writing a mix task that can just\nbe installed as a regular dependency in other projects, that mix task can use\ndependencies such as CliMate just as usual.\n\nCode generation is intended to be used by mix tasks that are best used as\nglobally installed tasks, or escripts, but that still need to be able to load\nmix projects (by loading `mix.exs` and projects code).\n\nFor regular applications that were using CliMate with either `use CliMate` or\n`CliMate.extend_cli()`, there is a single change to perform, that is replacing\nthat line with:\n\n```elixir\nrequire CliMate.CLI\nCliMate.CLI.extend()\n```","ref":"readme.html#migration-to-version-0-8-0"},{"type":"extras","title":"Migration to version 0.7.0 - Overview","doc":"The original version of CliMate included the CLI code in a consumer module,\nusing `use CliMate`. This allowed library authors to use CliMate in mix tasks\nthat could be installed by users with `mix archive.install hex some_package`.\nArchives installed that way cannot have dependencies so CliMate was providing a\nway to use it anyway.\n\nBut this solution had too many problems regarding code loading with the recent\nversions of Elixir. So we are stopping support for this feature.\n\nThe best way to provide commands with dependencies is to provide an\n[escript](https://hexdocs.pm/mix/main/Mix.Tasks.Escript.Build.html) or something\nlike [burrito](https://github.com/burrito-elixir/burrito).","ref":"readme.html#migration-to-version-0-7-0"},{"type":"extras","title":"Roadmap - Overview","doc":"* Merging code from Argument and Option for consistent capabilities\n* Support for subcommands in escripts","ref":"readme.html#roadmap"},{"type":"extras","title":"License - Overview","doc":"CliMate is released under the MIT License. See the [LICENSE](LICENSE.md) file for details.","ref":"readme.html#license"},{"type":"extras","title":"Getting Started","doc":"# Getting Started With CliMate\n\nThis guide demonstrates how to use CliMate to create command-line applications in Elixir. We'll walk through defining a simple command that outputs an integer in base 2 or base 8.","ref":"getting_started.html"},{"type":"extras","title":"Defining a Command - Getting Started","doc":"A command in CliMate consists of options, arguments, and metadata. First, declare your module as a Mix task and alias the `CliMate.CLI` module:\n\n```elixir\ndefmodule Mix.Tasks.Example do\n  alias CliMate.CLI\n  use Mix.Task\n\n  # ...\nend\n```\n\nThen define your command structure:\n\n```elixir\n@command name: \"mix example\",\n         module: __MODULE__,\n         options: [\n           verbose: [\n             short: :v,\n             type: :boolean,\n             default: false,\n             doc: \"Output debug info about the command.\"\n           ]\n         ],\n         arguments: [\n           n: [type: :integer, doc: \"The value to convert.\"],\n           base: [\n             cast: &__MODULE__.cast_base/1,\n             doc: \"A numeric base. Accepts 'two' or 'eight' only!\"\n           ]\n         ]\n```\n\nThe `name` should reflect how the command is invoked. This will be displayed in the help text.","ref":"getting_started.html#defining-a-command"},{"type":"extras","title":"Parsing Command Arguments - Getting Started","doc":"Use the `CliMate.CLI.parse_or_halt!/2` function to parse command arguments:\n\n```elixir\n@impl true\ndef run(argv) do\n  %{options: opts, arguments: args} = CLI.parse_or_halt!(argv, @command)\n\n  formatted = Integer.to_string(args.n, args.base)\n\n  if opts.verbose do\n    IO.puts(\"#{args.n} in base #{args.base} is #{formatted}\")\n  else\n    IO.puts(formatted)\n  end\nend\n\ndef cast_base(\"two\"), do: {:ok, 2}\ndef cast_base(\"eight\"), do: {:ok, 8}\ndef cast_base(_), do: {:error, :invalid_base}\n```\n\nThe `parse_or_halt!/2` function returns a map with `:options` and `:arguments` keys. Values are automatically cast according to the specified `:type` or `:cast` functions. If parsing fails, it will display the usage information and halt with an error message.","ref":"getting_started.html#parsing-command-arguments"},{"type":"extras","title":"Argument Types and Options - Getting Started","doc":"","ref":"getting_started.html#argument-types-and-options"},{"type":"extras","title":"Argument Types - Getting Started","doc":"CliMate supports the following argument types:\n\n- `:string` (default)\n- `:integer`\n- `:float`\n\n```elixir\narguments: [\n  count: [type: :integer, doc: \"A count parameter\"],\n  rate: [type: :float, doc: \"A floating point value\"],\n  name: [type: :string, doc: \"A name\"]\n]\n```","ref":"getting_started.html#argument-types"},{"type":"extras","title":"Arguments Configuration - Getting Started","doc":"Arguments support the following parameters:\n\n- `:required` - Default is `true`. Set to `false` for optional arguments.\n- `:cast` - A function or MFA tuple that casts the value.\n- `:doc` - Documentation string shown in help text.\n\n```elixir\narguments: [\n  required_arg: [],\n  custom_arg: [cast: &MyModule.custom_cast/1]\n  optional_arg: [required: false],\n]\n```\n\nNon-required arguments must be defined after required ones.","ref":"getting_started.html#arguments-configuration"},{"type":"extras","title":"Option Types - Getting Started","doc":"Options support the following types:\n\n- `:boolean` - Flags that don't need values (`--verbose`)\n- `:count` - Counts the number of times an option appears\n- `:integer` - Integer values\n- `:float` - Floating point values\n- `:string` - String values (default)\n\n```elixir\noptions: [\n  verbose: [type: :boolean],\n  count: [type: :count],\n  port: [type: :integer, default: 8080],\n  rate: [type: :float],\n  name: [] # :string is default\n]\n```","ref":"getting_started.html#option-types"},{"type":"extras","title":"Options Configuration - Getting Started","doc":"Options can be configured with:\n\n- `:short` - Single letter alias (`:v` for `-v`)\n- `:default` - Default value or function\n- `:keep` - Keep all values when an option is repeated\n- `:doc_arg` - Custom display name for the argument\n- `:doc` - Option description\n\n```elixir\noptions: [\n  names: [short: :n, keep: true, doc: \"Collects names (can be repeated)\"],\n  port: [short: :p, default: 4000, doc: \"Port number\"],\n  type: [doc_arg: \"db-type\", doc: \"Database type to use\"]\n]\n```","ref":"getting_started.html#options-configuration"},{"type":"extras","title":"Automatic Help Option - Getting Started","doc":"All commands automatically include a `--help` option. When used, it displays command usage and halts the program with a success code (0).\n\n```\n%> mix example --help\nmix example version 0.7.1\n\nSynopsis\n\n  mix example [options]    \n\nArguments\n\n  n\n  The value to convert.\n\n  base\n  A numeric base. Accepts 'two' or 'eight' only!\n\nOptions\n\n  -v, --verbose\n        Output debug info about the command. Defaults to false.\n\n      --help\n        Displays this help.\n```","ref":"getting_started.html#automatic-help-option"},{"type":"extras","title":"Providing Documentation for `mix help` - Getting Started","doc":"You can include command usage in your module documentation:\n\n```elixir\n@shortdoc \"Formats an integer in base two or eight\"\n\n@moduledoc \"\"\"\n#{@shortdoc}\n\n#{CliMate.CLI.format_usage(@command, format: :moduledoc)}\n\"\"\"\n```","ref":"getting_started.html#providing-documentation-for-mix-help"},{"type":"extras","title":"Advanced Usage - Getting Started","doc":"For more detailed information on configuring options and arguments, see:\n\n- `CliMate.CLI.Argument` for advanced argument configuration\n- `CliMate.CLI.Option` for advanced option configuration","ref":"getting_started.html#advanced-usage"}],"proglang":"elixir","content_type":"text/markdown","producer":{"name":"ex_doc","version":"0.37.3"}}